from . import libpyimgbuffer, libpysndbuffer


def copy_from(nparray, gformat, width=None, height=None):
    if type(nparray).__name__ == 'ndarray':
        if width is not None or height is not None:
            print('copy_from ignore width and height, because numpy array contains them.')
        buffer = GraphicBuffer()
        buffer.alloc_from_np(nparray, gformat)
        return buffer
    elif type(nparray).__name__ == 'bytes':
        if width is None or height is None:
            print('copy_from py bytes need width and height.')
            return None
        buffer = GraphicBuffer()
        buffer.alloc_from_bytes(nparray, width, height, gformat)
        return buffer
    else:
        print('copy_from only support numpy array, but argument is ' + str(type(nparray)))
        return None


class GraphicBuffer:
    def __init__(self, instance=None):
        self.instance = instance

    def alloc(self, width, height, gformat, cma=False):
        cma = 0 if not cma else 1
        self.instance = libpyimgbuffer.create(width, height, gformat, cma)

    def alloc_from_np(self, ndarray, gformat):
        self.instance = libpyimgbuffer.create_from_array(ndarray, gformat)

    def alloc_from_bytes(self, bytes, width, height, gformat):
        self.instance = libpyimgbuffer.create_from_bytes(bytes, width, height, gformat)

    def __del__(self):
        libpyimgbuffer.delete(self.instance)

    def size(self):
        return libpyimgbuffer.validsize(self.instance)

    def asarray(self):
        return libpyimgbuffer.to_array(self.instance)

    def width(self):
        return libpyimgbuffer.width(self.instance)

    def height(self):
        return libpyimgbuffer.height(self.instance)

    def resize(self, width, height):
        img = libpyimgbuffer.resize(self.instance, width, height)
        dst = GraphicBuffer(img)
        return dst

    def crop(self, x, y, width, height):
        img = libpyimgbuffer.crop(self.instance, x, y, width, height)
        dst = GraphicBuffer(img)
        return dst

    def rotate(self, degree):
        if degree not in [90, 180, 270]:
            print('Unsupport this degree %d, only support (90, 180, 270)' % degree)
        img = libpyimgbuffer.rotate(self.instance, degree)
        dst = GraphicBuffer(img)
        return dst

    def convert(self, gformat):
        img = libpyimgbuffer.convert(self.instance, gformat)
        dst = GraphicBuffer(img)
        return dst


class SoundBuffer:
    def __init__(self, instance=None):
        self.instance = instance

    def __del__(self):
        libpysndbuffer.delete(self.instance)
