from . import libpydisplay, libpydrmdisplay
from .buffer import GraphicBuffer
from .defines import *
from .error import *


class Display:
    def __init__(self, window, want_width=0, want_height=0, fullscreen=False, display_port=DISP_WINDOW, maxfps=60):
        if display_port == DISP_WINDOW:
            if want_width == 0 or want_height == 0:
                want_width = 640
                want_height = 360
        self.instance = libpydisplay.create(window, display_port, want_width, want_height, fullscreen, maxfps)
        self.error = 0

    def __del__(self):
        libpydisplay.delete(self.instance)

    def addview(self, x, y, w, h):
        x, y, w, h = self.__transform__(x, y, w, h)
        return libpydisplay.addview(self.instance, x, y, w, h)

    def rmview(self, index):
        return libpydisplay.rmview(self.instance, index)

    def mvview(self, index, x, y, w, h):
        x, y, w, h = self.__transform__(x, y, w, h)
        return libpydisplay.mvview(self.instance, index, x, y, w, h)

    def imshow(self, graphic_buffer, view=None):
        if graphic_buffer is None:
            print('imshow: frame is none !')
            return RET_INVALID_ARGUMENT
        if view is None:
            view = -1
        ret = libpydisplay.imshow(self.instance, graphic_buffer.instance, view)
        if ret < 0:
            return RET_TERMINATED
        else:
            return RET_SUCCESS

    @property
    def width(self):
        return libpydisplay.width(self.instance)

    @property
    def height(self):
        return libpydisplay.height(self.instance)

    # def read(self, width=0, height=0, gformat=FMT_RGB888):
    #     img = libpydisplay.read(self.instance, width, height, gformat)
    # if img is None:
    #     self.error = TOY_TRY_AGAIN
    #     dst = None
    # else:
    #     dst = GraphicBuffer(img)
    #     return self.error, dst

    def __transform__(self, x, y, w, h):
        # Transform coordinate system -> LT(0,0), RB(W,H)
        # print(f"__transform__ {x},{y},{w},{h} -> {x},{self.height-y-h},{w},{h}")
        return x, self.height - y - h, w, h

    def dump(self):
        libpydisplay.dump()


class DrmDisplay:
    def __init__(self, display_port=DISP_HDMI_A_1, zpos=3, want_width=0, want_height=0):
        self.instance = libpydrmdisplay.create(display_port, zpos, want_width, want_height)
        self.error = 0

    def __del__(self):
        libpydrmdisplay.delete(self.instance)

    def imshow(self, graphic_buffer, view=None):
        if graphic_buffer is None:
            print('imshow: frame is none !')
            return RET_INVALID_ARGUMENT
        if view is None:
            view = -1
        libpydrmdisplay.imshow(self.instance, graphic_buffer.instance, view)
        return RET_SUCCESS

    @property
    def width(self):
        return libpydrmdisplay.width(self.instance)

    @property
    def height(self):
        return libpydrmdisplay.height(self.instance)
