from . import libpyrtspclient
from . import libpyrtspserver
from .buffer import GraphicBuffer
from .defines import *
from .error import *


class RtspCapture:
    def __init__(self, url, user=None, password=None, use_tcp=False):
        self.instance = libpyrtspclient.create(url, user, password, use_tcp)
        self.error = 0

    def __del__(self):
        libpyrtspclient.delete(self.instance)

    def read(self, width=0, height=0, gformat=FMT_RGB888):
        img = libpyrtspclient.read(self.instance, width, height, gformat)
        if img is None:
            self.error = RET_TRY_AGAIN
            dst = None
        else:
            dst = GraphicBuffer(img)
        return self.error, dst


class RtspWriter:
    def __init__(self, method, encoder=FMT_H264, port=554):
        self.instance = libpyrtspserver.create(method, encoder, port)

    def __del__(self):
        libpyrtspserver.delete(self.instance)

    def write(self, frame, width=0, height=0):
        libpyrtspserver.write(self.instance, frame.instance, width, height)
